from .console.cnsapp import console_app, tool, proxy_tool

'''
Ramona - Runtime supervisor
===========================

Project homepage: http://ateska.github.com/ramona/
Project page on PyPi: http://pypi.python.org/pypi/ramona
Project page on GitHub: https://github.com/ateska/ramona 

Authors
-------
Ales Teska
Jan Stastny

License
-------
BSD 2-clause "Simplified" License
'''

#For proper version formatting see:
# - http://www.python.org/dev/peps/pep-0386/
# - http://www.python.org/dev/peps/pep-0345/#version-specifiers

version = '1.1b1' # Also in setup.py
